
function [f,g,f1,g1,table_ends_degrees_extended]=...
    define_domain_chebfun(domain_type,tol)

%--------------------------------------------------------------------------
% GIVEN A DOMAIN SELECTED BY domain_type THE BOUNDARY IS DEFINED
% PARAMETRICALLY AS (f,g) WHERE f AND g ARE CHEBFUNS. THE BOUNDARY (f*,g*)
% IS APPROXIMATED SO THAT
%
%                         \|f-f*\| <= tol * \|f*\|
%
%                          \|g-g*\| <= tol * \|g*\|
%
% THE TANGENT VECTORS (f1,g1) ARE ALSO COMPUTED.
% THE SINGULAR POINTS ARE ORDERED IN INCREASING ORDER BY THE ABSCISSA t OF
% EACH PARAMETRICAL FUNCTION f AND g AND THE DEGREE OF EACH POLYNOMIAL IS
% STORED IN THE VECTOR table_ends_degrees_extended THAT DETERMINES THE k-th
% INTERVAL IN THE VARIABLE t AS
%
% [table_ends_degrees_extended(k) table_ends_degrees_extended(k+1)]
%
% AND THE DEGREES OF THE PIECEWISE POLYNOMIALS f, g IN SUCH INTERVAL AS
%
%                       table_ends_degrees_extended(k)
%
%                      table_ends_degrees_extended(k+1)
%
% RESPECTIVELY.
%--------------------------------------------------------------------------
%% Copyright (C) 2007-2010 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: June 15, 2007 - March 30, 2011.
%--------------------------------------------------------------------------

% SETTING CHEBFUN TOLERANCE.
chebfunpref('eps',tol);

% DEFINING DOMAIN.
switch domain_type
    case 1 % Disk.
        f=chebfun('cos(t)',[0 2*pi]);
        g=chebfun('sin(t)',[0 2*pi]);

    case 2 % Lune.
        f=chebfun('0.5*(cos(t)+1)','0.5*cos((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
        g=chebfun('0.5*(sin(t)+1)','0.5*sin((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);

    case 3 % Union of two disks.
        f=chebfun('cos(t)+sqrt(2)/2','cos(t-pi/2)-sqrt(2)/2',...
            [-3*pi/4 3*pi/4 9/4*pi]);
        g=chebfun('sin(t)','sin(t-pi/2)', [-3*pi/4 3*pi/4 9/4*pi]);

    case 4 % Intersection of two disks.
        f=chebfun('cos(t)-sqrt(2)/2+1/2','cos(t+pi/2)+1/2+sqrt(2)/2',...
            [-pi/4 pi/4 3/4*pi]);
        g=chebfun('1/2+sin(t)','1/2+sin(t+pi/2)', [-pi/4 pi/4 3/4*pi]);

    case 5 % Cardioid.
        description_type=1;
        switch description_type
            case 1
                f=chebfun('cos(t).*(1-cos(t))+1',[0 2*pi]);
                g=chebfun('sin(t).*(1-cos(t))',[0 2*pi]);
            case 2
                f=chebfun('cos(t).*(1-cos(t))+1',[pi/2 2*pi+pi/2]);
                g=chebfun('sin(t).*(1-cos(t))',[pi/2 2*pi+pi/2]);
        end

    case 6 % Deltoid (a=3, b=1 in Wolfram MathWorld definition).
        f=chebfun('2*1*cos(t)+1*cos(2*t)',[0 2*pi]); % 2*b*cos(t)+b*cos(2*t)
        g=chebfun('2*1*sin(t)-1*sin(2*t)',[0 2*pi]); % 2*b*sin(t)-b*sin(2*t)

end

% COMPUTE TANGENT VECTORS.
f1=diff(f);
g1=diff(g);


% DETERMINING END POINTS AND POLYNOMIAL DEGREES OF f AND g. USING COLUMN
% VECTORS. IMPORTANT: STARTING FROM THE INITIAL POINT OF THE
% PARAMETRISATION AND SETTING AS f,g DEGREES THOSE OF THE INITIAL
% PARAMETRISATION INTERVAL.
fends=(f.ends)';
ffuns=(f.funs)';

for index=1:length(fends)-1
    localdegree=ffuns(index);
    fd(index,1)=length(localdegree)-1;
    fef(index,1)=fends(index+1);
end

fd=[fd(1); fd];

gends=(g.ends)';
gfuns=(g.funs)';

for index=1:length(gends)-1
    localdegree=gfuns(index);
    gd(index,1)=length(localdegree)-1;
    gef(index,1)=gends(index+1);
end

gd=[gd(1); gd];

% STORE END POINTS AND POLYNOMIAL DEGREES OF f AND g IN A TABLE,
% ORDERING END POINTS AND AVOIDING MULTIPLE OCCURRENCES.
table_ends_degrees_extended=discontinuities_manager(fends,fd,gends,gd);


