
function table_ends_degrees=discontinuities_manager(fends,fdegree,gends,...
    gdegree)

%--------------------------------------------------------------------------
% PURPOSE. 
%-------------------
%
% THE BOUNDARY OF A CLOSED BIVARIATE JORDAN-DOMAIN "OMEGA" IS DEFINED
% PARAMETRICALLY IN TERMS OF TWO PIECEWISE CHEBFUNS AS (f,g). 
% ONCE THAT WE HAVE AT HAND THE ENDPOINTS fends, gends OF f AND g 
% RESPECTIVELY, AS WELL AS THEIR DEGREES IN ANY SUBINTERVAL 
% (fends(j),fends(j+1)), (gends(k),gends(k+1)), WE SORT THEM IN INCREASING
% ORDER, DETERMINING A TABLE table_ends_degrees.
%
% EXAMPLE:
%
% >> fends=[0.2 0.5 2*pi]';
% >> gends=[0.3 0.4 0.7 2*pi]';
% >> fdegree=[3 5 4]';
% >> gdegree=[2 8 3 5]';
% >> table_ends_degrees=discontinuities_manager(fends,fdegree,gends,...
%    gdegree)
%
% table_ends_degrees =
%
%    0.2000    3.0000    2.0000
%    0.3000    5.0000    2.0000
%    0.4000    5.0000    8.0000
%    0.5000    5.0000    3.0000
%    0.7000    4.0000    3.0000
%    6.2832    4.0000    5.0000
%
%--------------------------------------------------------------------------
% REFERENCE PAPERS. 
%-------------------
%
% 1. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-like and triangulation-free cubature over polygons".
%
% 2. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-Green cubature over spline curvilinear polygons"
%
%--------------------------------------------------------------------------
% INPUT:
%--------
%
% fends:    a column vector that contains "f" piecewise chebfun endpoints.
% fdegree:  a column vector that contains "f" piecewise chebfun degrees.
% gends:    a column vector that contains "g" piecewise chebfun endpoints.
% gdegree:  a column vector that contains "g" piecewise chebfun degrees.
%
%---------
% OUTPUT:
%---------
%
% table_ends_degrees: n x 3 matrix containing intervals, f degrees and g 
%     degrees as described above.
%
%--------------------------------------------------------------------------
%% Copyright (C) 2007-2010 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: June 15, 2007 - February 12, 2010.
%--------------------------------------------------------------------------

fef=fends(1:length(fends));
gef=gends(1:length(gends));

table_ends_degrees=[];

loc_fef=fef;
loc_gef=gef;

loc_fd=fdegree;
loc_gd=gdegree;

while length(loc_fef)+length(loc_gef) > 0
    
    if (loc_fef(1) < loc_gef(1))
            table_ends_degrees=[table_ends_degrees; ...
                loc_fef(1) loc_fd(1) loc_gd(1)];
            
            loc_fef=loc_fef(2:end);
            loc_fd=loc_fd(2:end);
            
    else
        if (loc_gef(1) < loc_fef(1))
            table_ends_degrees=[table_ends_degrees; ...
                loc_gef(1) loc_fd(1) loc_gd(1)];
            
            loc_gef=loc_gef(2:end);
            loc_gd=loc_gd(2:end);
        else
            table_ends_degrees=[table_ends_degrees; ...
                loc_fef(1) loc_fd(1) loc_gd(1)];
            
            loc_fef=loc_fef(2:end);
            loc_fd=loc_fd(2:end);
            
            loc_gef=loc_gef(2:end);
            loc_gd=loc_gd(2:end);
            
        end
    end
    
end

