
%--------------------------------------------------------------------------
% THIS MAIN MATLAB CODE TESTS ON SEVERAL BIVARIATE DOMAINS SPECIFIED IN THE
% m-FILE define_domain_chebfun.m THE CUBATURE OF FUNCTIONS DEFINED IN THE
% m-FILE fct2D.m VIA GREEN FORMULA AND LEGENDRE TENSORIAL RULE ON THE
% SQUARE. MODIFYING THE VARIABLE cubature_type SEVERAL RULES THAT PROVIDES
% A CUBATURE RULE ON A RECTANGLE CAN BE USED FOR COMPUTING CUBATURE ON THE
% CURVILINEAR DOMAIN.
%--------------------------------------------------------------------------
% REFERENCE PAPERS.
%-------------------
%
% 1. A. SOMMARIVA and M. VIANELLO
%    "Gauss-like and triangulation-free cubature over polygons".
%
% 2. A. SOMMARIVA and M. VIANELLO
%    "Gauss-Green cubature over spline curvilinear polygons"
%
%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES :
%----------------------
%
% 1. define_domain_chebfun
% 2. splinegauss_chebfun_2010
%
% THESE ROUTINES ARE EXTERNAL TO THIS DEMO FILE.
%--------------------------------------------------------------------------
%% Copyright (C) 2008 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: Nov. 2, 2009 - March. 30, 2011.
%--------------------------------------------------------------------------

clear all;

%-----------------------------------------------------------------------
% domain_type :
%----------------------
%
% The domain is defined in "define_domain_chebfun.m". You can add Your own
% domain by adding in "define_domain_chebfun.m" your own domain via its
% boundary in parametric form.
%
% The value of "domain_type" correspond to the following choices:
%
% 1. Disk.
%   f=chebfun('cos(t)',[0 2*pi]); g=chebfun('sin(t)',[0 2*pi]);
%
% 2. Lune.
%    f=chebfun('0.5*(cos(t)+1)','0.5*cos((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%    g=chebfun('0.5*(sin(t)+1)','0.5*sin((3*pi/2)-t)',[-pi/2 pi 3*pi/2]);
%
% 3. Union of two disks.
%    f=chebfun('cos(t)+sqrt(2)/2','cos(t-pi/2)-sqrt(2)/2',...
%            [-3*pi/4 3*pi/4 9/4*pi]);
%    g=chebfun('sin(t)','sin(t-pi/2)', [-3*pi/4 3*pi/4 9/4*pi]);
%
% 4. Intersection of two disks.
%    f=chebfun('cos(t)-sqrt(2)/2+1/2','cos(t+pi/2)+1/2+sqrt(2)/2',...
%            [-pi/4 pi/4 3/4*pi]);
%    g=chebfun('1/2+sin(t)','1/2+sin(t+pi/2)', [-pi/4 pi/4 3/4*pi]);
%
% 5. Cardioid.
%    f=chebfun('cos(t).*(1-cos(t))+1',[0 2*pi]);
%    g=chebfun('sin(t).*(1-cos(t))',[0 2*pi]);
%
% 6. Deltoid.
%    f=chebfun('2*1*cos(t)+1*cos(2*t)',[0 2*pi]); % 2*b*cos(t)+b*cos(2*t);
%    g=chebfun('2*1*sin(t)-1*sin(2*t)',[0 2*pi]); % 2*b*sin(t)-b*sin(2*t).
%
%-----------------------------------------------------------------------
domain_type=6;

%--------------------------------------------------------------------------
% N :
%-----
%
% Degree of precision of the algebraic cubature rule.
%--------------------------------------------------------------------------
N=11;

%--------------------------------------------------------------------------
% rotation, P, Q :
%----------------------
%
% Choosing a baseline (segment joining P and Q), one can provide a cubature
% rule with all the points inside the domain.
%
% 0: no rotation.
% 1: rotation decided automatically. Good for convex polygons.
% 2. rotation provided by the user.
%--------------------------------------------------------------------------

rotation=2;

switch domain_type
    case 1
        P=[0 0]; Q=[0 1]; %Disk.
    case 2
        P=[0.5 0]; Q=[0.5 1]; % Lune (good baseline).
        P=[0 0]; Q=[0 1]; % Lune (bad baseline).
    case 3
        P=[-1 0]; Q=[1 0]; % Union of two disks.
    case 4
        P=[0.5 0]; Q=[0.5 1]; % Intersection of two disks.
    case 5
        P=[0.25 0]; Q=[0.25 1]; %Cardioid.
    case 6
        P=[-1.5 0]; Q=[3 0]; % Deltoid.
    otherwise
        P=[0 0]; Q=[0 1];
end

%--------------------------------------------------------------------------
% cubature_type :
%----------------------
%
% The cubature is based on rules over a rectangle. We used the following
% examples.
%
% 0: Padua points.
% 1: Fejer 1 type (tensorial).
% 2: Fejer 2 type (tensorial).
% 3: Clenshaw-Curtis (tensorial).
% 4: Gauss-Legendre (tensorial).
% 5: Gauss-Legendre-Lobatto (tensorial).
%
% Use as default "cubature_type = 4" that correspond to a Gauss-Legendre
% tensorial formula (as in the paper). The other cubature formulae are
% given just for sake of completeness.
%--------------------------------------------------------------------------

cubature_type=4;
reference_cubature_type=4;   % CHOOSING THE CUBATURE TYPE FOR REFERENCE
% RESULT.

%--------------------------------------------------------------------------
% tol :
%----------------------
%
% Chebfun tolerance. Used to describe the boundary.
%--------------------------------------------------------------------------

tol=eps;
tol_ref=eps;



% ------------ MAIN PROGRAM STARTS HERE -----------------------------------




%--------------------------------------------------------------------------
% CHEBFUNGAUSS
%--------------------------------------------------------------------------
% DEFINING DOMAIN.
[f,g,f1,g1,table_ends_degrees_extended]=...
    define_domain_chebfun(domain_type,tol);

% table_ends_degrees_extended=[fends(1) fd(1) gd(1); table_ends_degrees]
% TABLE OF END POINTS, f,g DEGREES.
fends=table_ends_degrees_extended(:,1);
table_ends_degrees=table_ends_degrees_extended(2:end,:);

% CONSIDERING END POINTS TO MAKE "BLOCKS".
polygon_sides=[feval(f,fends(1)) feval(g,fends(1)); ...
    feval(f,table_ends_degrees(:,1)) feval(g,table_ends_degrees(:,1))];

fprintf('\n \t %% >>CHEBFUNGAUSS (DEG. %5.0f)', N);

cputimes(1)=cputime;

% DETERMINING CUBATURE POINTS AND WEIGHTS.
[nodes_x,nodes_y,weights]=splinegauss_chebfun_2010(N,f,g,f1,g1,...
    polygon_sides,rotation,P,Q,table_ends_degrees_extended,...
    cubature_type);

%--------------------------------------------------------------------------
% STATS.
%--------------------------------------------------------------------------
fprintf('\n \n \t -------------------------------------------------------')
fprintf('\n \t              CHEBFUN BOUNDARY PROPERTIES')
fprintf('\n \t -------------------------------------------------------')

fprintf('\n \n \t [BOUNDARY CHEBFUN TOL.]: %2.2e \n ', tol);


for index=1:size(table_ends_degrees_extended,1)-1
    fprintf('\n \t [PT1]: %3.4f [PT3]: %3.4f [Deg(f)]: %3.0f [Deg(g)]: %3.0f',...
        table_ends_degrees_extended(index,1),...
        table_ends_degrees_extended(index+1,1),...
        table_ends_degrees_extended(index,2),...
        table_ends_degrees_extended(index,3));


end


fprintf('\n \n \t -------------------------------------------------------')
fprintf('\n \t                     CHEBFUN GAUSS')
fprintf('\n \t -------------------------------------------------------')

fprintf('\n \t [ADE]: %5.0f [NODES]: %10.0f',N,...
    length(nodes_x));

switch cubature_type

    % 0: Padua points.
    % 1: Fejer 1 type (tensorial).
    % 2: Fejer 2 type (tensorial).
    % 3: Clenshaw-Curtis (tensorial).
    % 4: Gauss-Legendre (tensorial).
    % 5: Gauss-Legendre-Lobatto (tensorial).

    case 0
        fprintf('\n \n \t [PADUA TYPE CUBATURE]');
    case 1
        fprintf('\n \n \t [FEJER TYPE 1 TENSORIAL CUBATURE]');
    case 2
        fprintf('\n \n \t [FEJER TYPE 2 TENSORIAL CUBATURE]');
    case 3
        fprintf('\n \n \t [CLENSHAW-CURTIS TENSORIAL CUBATURE]');
    case 4
        fprintf('\n \n \t [GAUSS-LEGENDRE TENSORIAL CUBATURE]');
    case 5
        fprintf('\n \n \t [GAUSS-LEGENDRE-LOBATTO TENSORIAL CUBATURE]');

end

% TESTING NUMBER OF CUBATURE NODES IN THE GAUSSIAN CASE.
if (cubature_type == 4)
    for index=1:size(table_ends_degrees,1)
        Ntau=floor(N/2)+1;
        ss=(2*Ntau-1)*max(table_ends_degrees(index,2),table_ends_degrees(index,3))+...
            (table_ends_degrees(index,2)+table_ends_degrees(index,3))-1;
        Nu=floor(ss/2)+1;

        number_pts(index)=Ntau*Nu;
    end
    ss=sum(number_pts);

    fprintf('\n \t [THEORETICAL NUMBER OF NODES]: %10.0f',ss);
end


% PLOTTING DOMAIN AND CUBATURE NODES.

clf;
hold on

plot(nodes_x,nodes_y,'r*','MarkerSize',3);
plot(f,g,'k-','LineWidth',2);

axis square;
box on;
hold off;


fprintf('\n \n');