
function [nodes_x,nodes_y,weights]=splinegauss_chebfun_2010(N,f,g,f1,g1,...
    polygon_sides,rotation,P,Q,table_ends_degrees_extended,cubature_type)

%--------------------------------------------------------------------------
% REFERENCE PAPERS. 
%-------------------
%
% 1. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-like and triangulation-free cubature over polygons".
%
% 2. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-Green cubature over spline curvilinear polygons"
%
%--------------------------------------------------------------------------
% INPUT:
%--------
%
%
% [N]      : DEGREE OF PRECISION OF THE ALGEBRAIC CUBATURE RULE.
%
% [f,g,f1,g1]: CHEBFUNS DESCRIVING THE (CLOSED) BOUNDARY.
%
% [polygon_sides]: IF THE POLYGON HAS "L" SIDES, "boundary.pts" IS A 
%            VARIABLE CONTAINING ITS VERTICES, ORDERED COUNTERCLOCKWISE. 
%            AS LAST ROW MUST HAVE THE COMPONENTS OF THE FIRST VERTEX.
%            IN OTHER WORDS, THE FIRST ROW AND LAST ROW ARE EQUAL.
%            "polygon_sides" IS A "L+1 x 2" MATRIX. IT IS USED ONLY TO MAKE
%            A SUITABLE ROTATION OF THE DOMAIN (IF REQUIRED BY THE USER).
%
% [rotation,P,Q]: Choosing a baseline (segment joining P and Q), one can  
% provide a cubature rule with all the points inside the domain.
%
%            0: no rotation.
%            1: rotation decided automatically. Good for convex polygons.
%            2. rotation provided by the user.
%
% [table_ends_degrees_extended]: matrix M x 3 where the first column
%            consists of the end points, the second the degree of the 
%            polynomial in x, the second in y. The initial point is also 
%            stored. 
%
% [cubature_type]: 0: PADUA POINTS.
%                  1: FEJER 1 TYPE (TENSORIAL).
%                  2: FEJER 2 TYPE (TENSORIAL).
%                  3: CLENSHAW-CURTIS (TENSORIAL).
%                  4: GAUSS-LEGENDRE (TENSORIAL).
%
%---------
% OUTPUT:
%---------
%
% [nodes_x, nodes_y]: THE CUBATURE RULE PRODUCES THE NODES 
%            "(nodes_x,nodes_y)". IF THE CUBATURE RULE GENERATES "K" NODES,
%            "nodes_x" AND "nodes_y" ARE "K x 1" COLUMN VECTORS.
%
% [weights]: cubature weights.
%
%--------------------------------------------------------------------------
%% Copyright (C) 2007-2010 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: June 15, 2007 - February 21, 2010.
%--------------------------------------------------------------------------


%----------------------------------------------------------------------
% BOUNDARY PTS.
%----------------------------------------------------------------------
x_bd=polygon_sides(:,1);
y_bd=polygon_sides(:,2);

%----------------------------------------------------------------------
% "MINIMUM" RECTANGLE CONTAINING POLYGON.
%----------------------------------------------------------------------
x_min=min(x_bd); x_max=max(x_bd);
y_min=min(y_bd); y_max=max(y_bd);

%--------------------------------------------------------------------------
% POLYGON ROTATION (IF NECESSARY).
%--------------------------------------------------------------------------

switch rotation
    case 0
        % NO ROTATION.
        rot_matrix=eye(2);
        axis_abscissa=[x_min 0];  
        % NON REALLY AN AXIS. IT IS IMPORTANT TO HAVE THE MINIMUM VALUE OF
        % THE ABSCISSAS OF THE POLYGON.
    case 1
        % AUTOMATIC ROTATION.
        [polygon_sides,rot_matrix,rot_angle,axis_abscissa,P,Q]=...
            auto_rotation(polygon_sides,[],[]);
    case 2
        % PREFERRED ROTATION.
        nrm_vect=norm(Q-P);
        if (nrm_vect > 0)
            direction_axis=(Q-P)/nrm_vect;
            [polygon_sides,rot_matrix,rot_angle,axis_abscissa,P,Q]=...
                auto_rotation(polygon_sides,P,Q);
        else
            [polygon_sides,rot_matrix,rot_angle,axis_abscissa,P,Q]=...
                auto_rotation(polygon_sides,P,Q);
        end
end

% Notation: "xi" as in the paper [1].
xi=axis_abscissa(1);

%--------------------------------------------------------------------------
% COMPUTE NODES.
%--------------------------------------------------------------------------

nodes_x=[];
nodes_y=[];
weights=[];

% Number of Blocks.
L=size(table_ends_degrees_extended,1)-1;

for block_index=1:L

    % Initial and final indices of "control points" in the block.

    % Polynomial degree in the block.
    polynomial_block_degree_1=table_ends_degrees_extended(block_index+1,2);
    polynomial_block_degree_2=table_ends_degrees_extended(block_index+1,3);
    
    % Control points (x_loc,y_loc) in the block.
    x_loc=polygon_sides(block_index:block_index+1,1);
    y_loc=polygon_sides(block_index:block_index+1,2);


    % Parametrical description of the block.
    s_loc=[table_ends_degrees_extended(block_index,1); ...
        table_ends_degrees_extended(block_index+1,1)];

    % Cubature rule on the square [-1,1] x [-1,1].
    % Padua Points: 0, Gauss-Legendre: 4.
    [x_pts, y_pts, wpd]=cubature_manager(N,polynomial_block_degree_1,...
        polynomial_block_degree_2,cubature_type);
    
    %-----------------------------------------
    % COMPUTING THE NODES.
    %-----------------------------------------

    % Computing "q_ij_u" (see formula (19)).

    half_pt_s=(s_loc(1)+s_loc(2))/2;
    half_length_s=(s_loc(2)-s_loc(1))/2;
    q_ij_u=half_pt_s+half_length_s*y_pts;

    % Evaluating S_i1(q_ij(u)), S_i2(q_ij(u)).
    S_i1_q_ij_u=feval(f,q_ij_u);
    S_i2_q_ij_u=feval(g,q_ij_u);
    
    S_i_q_ij_u_rot=rot_matrix*[S_i1_q_ij_u'; S_i2_q_ij_u'];
    S_i1_q_ij_u=(S_i_q_ij_u_rot(1,:))';
    S_i2_q_ij_u=(S_i_q_ij_u_rot(2,:))';
    
    % See formula (18). Terms involving the spline in the arguments of "f".
    scaling_fact_plus=(S_i1_q_ij_u+xi)/2;
    scaling_fact_minus=(S_i1_q_ij_u-xi)/2;

    x=scaling_fact_minus.*x_pts+scaling_fact_plus;
    y=S_i2_q_ij_u;

    % Applying rotation, if required.
    rot_pts=[x y];
    rot_pts=rot_pts*rot_matrix;

    x_rot=rot_pts(:,1);
    y_rot=rot_pts(:,2);

    nodes_x=[nodes_x; x_rot];
    nodes_y=[nodes_y; y_rot];

    %-----------------------------------------
    % COMPUTING THE WEIGHTS.
    %-----------------------------------------    
    
    f1=diff(f);
    S1_i1_q_ij_u=feval(f1,q_ij_u);
    S1_i2_q_ij_u=feval(g1,q_ij_u);
    
    S1_i_q_ij_u=[S1_i1_q_ij_u'; S1_i2_q_ij_u'];
    S1_i_q_ij_u_rot=rot_matrix*S1_i_q_ij_u;
    S1_i2_q_ij_u=(S1_i_q_ij_u_rot(2,:))';
    
    
    initial_scaling_factor=(s_loc(2)-s_loc(1))/4; % SCALING FACTOR.
    
    local_weights=initial_scaling_factor.*(2*scaling_fact_minus).*...
        S1_i2_q_ij_u.*wpd;

    weights=[weights; local_weights];


end





%--------------------------------------------------------------------------
% FUNCTIONS USED BY THE ROUTINE splinegauss_chebfun_2010.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% 1. "paduapts".
%--------------------------------------------------------------------------

function [varargout] = pdcub(n,xyrange,varargin)

%--------------------------------------------------------------------------
% USAGE of "pdcub".
%
% PadL = pdcub(n,xyrange)
% [cubature,PadL] = pdcub(n,xyrange,funct,opt1,opt2,...)
%
% This function computes the Padua points defined in the rectangle
% [xyrange(1),xyrange(2)] x [xyrange(3),xyrange(4)] and the
% corresponding cubature weights as a matrix of abscissas (first
% column), ordinates (second column) and weights (third column).
% Optionally, it computes the cubature of the function f (optional
% input argument).
%--------------------------------------------------------------------------
% INPUT.
%
% n        : interpolation degree
% xyrange  : a vector [a,b,c,d] defining the rectangle [a,b] x [c,d]
% funct    : function to be interpolated in the form
%            funct(x,y,opt1,opt2,...), where opt1, opt2, ... are
%            optional arguments for funct
%
% OUTPUT.
%
% PadL     : a matrix with the abscissas of Padua points in the
%            first column, the ordinates in the second and the
%            cubature weights in the third
% cubature : cubature of the integrand funct
%--------------------------------------------------------------------------
% FUNCTIONS CALLED BY THIS CODE:
% 1. pdpts
% 2. pdwtsMM
%--------------------------------------------------------------------------
% EXAMPLES.
%
% 1)
% Compute the Padua points and the cubature weights of degree 50
%
% xyrange = [0,1,0,1];
% PadL = pdcub(50,xyrange);
%
% 2)
% Compute the cubature of the Franke's function defined in funct.m
%
% xyrange = [0,1,0,1];
% [cubature,PadL] = pdcub(50,xyrange,@f);
%--------------------------------------------------------------------------
% PROCEDURES USED BY THE CODE:
% 1. pdpts
% 2. pdwtsMM
%--------------------------------------------------------------------------
% Copyright (C) 2008-2009
% Marco Caliari, Stefano De Marchi, Alvise Sommariva, Marco Vianello.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%
% Author:
%          Marco Caliari     <marco.caliari@univr.it>
%          Stefano De Marchi <stefano.demarchi@univr.i>
%          Alvise Sommariva  <alvise@euler.math.unipd.it>
%          Marco Vianello    <marcov@euler.math.unipd.it>
%
% Date: February 09, 2010.
%--------------------------------------------------------------------------

% Compute the Padua points in the rectangle defined by xyrange
PadL = pdpts(n,xyrange);
% Compute the cubature weights
PadL(:,3) = pdwtsMM(n,xyrange);
if (nargin < 2)
    error('Too few input arguments')
elseif (nargin >= 3)
    funct = varargin{1};
    varargout{1} = PadL(:,3)'*feval(funct,PadL(:,1),PadL(:,2),...
        varargin{2:end});
    varargout{2} = PadL;
else
    varargout{1} = PadL;
end


function [varargout] = pdwtsMM(n,varargin)

%--------------------------------------------------------------------------
% USAGE of "pdwtsMM".
%
% L = pdwtsMM(n)
% L = pdwtsMM(n,xyrange)
% [L1,L2,L] = pdwtsMM(n)
% [L1,L2,L] = pdwtsMM(n,xyrange)
%
% Compute the cubature weights L so that, if Pad is the
% matrix of Padua points computed through the call
% Pad = pdpts(n) or Pad = pdpts(n,xyrange), then the cubature of
% the function f is given by L'*f(Pad(:,1),Pad(:,2)).
% Otherwise, one can compute the cubature weights L1 and L2, so
% that, if X1,Y1 and X2,Y2 are the subgrids of Padua points
% computed through the call [X1,Y1,X2,Y2] = pdpts(n) or
% [X1,Y1,X2,Y2] = pdpts(n,xyrange), then the cubature of the
% function f is given by
% sum(sum(L1.*f(X1,Y1)))+sum(sum(L1.*f(X2,Y2))).
%--------------------------------------------------------------------------
% INPUT.
%
% n       : interpolation degree
% xyrange : an optional vector [a,b,c,d] defining the rectangle
%           [a,b] x [c,d]. By default, xyrange = [-1,1,-1,1]
%
% OUTPUT.
%
% L       : cubature weights associated to the matrix Pad of
%           Padua points
% L1,L2   : cubature weights associated to the subgrids
%           X1,Y1 and X2,Y2 of Padua poins.
%--------------------------------------------------------------------------
% FUNCTIONS CALLED BY THIS CODE:
% no external function is used.
%--------------------------------------------------------------------------

% Copyright (C) 2008-2009
% Marco Caliari, Stefano De Marchi, Alvise Sommariva, Marco Vianello.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%
% Author:
%          Marco Caliari     <marco.caliari@univr.it>
%          Stefano De Marchi <stefano.demarchi@univr.i>
%          Alvise Sommariva  <alvise@euler.math.unipd.it>
%          Marco Vianello    <marcov@euler.math.unipd.it>
%
% Date: March 04, 2009.
%--------------------------------------------------------------------------

if (nargin == 1)
    xyrange = [-1,1,-1,1];
else
    xyrange = varargin{1};
end
if (n == 0)
    % degree 0
    L1 = (xyrange(2)-xyrange(1))*(xyrange(4)-xyrange(3));
    L2 = zeros(1,0);
    L = L1;
else
    argn = linspace(0,pi,n+1);
    argn1 = linspace(0,pi,n+2);
    k = [0:2:n]';
    l = (n-mod(n,2))/2+1;
    % even-degree Chebyshev polynomials on the subgrids
    TE1 = cos(k*argn(1:2:n+1));
    TE1(2:l,:) = TE1(2:l,:)*sqrt(2);
    TO1 = cos(k*argn(2:2:n+1));
    TO1(2:l,:) = TO1(2:l,:)*sqrt(2);
    TE2 = cos(k*argn1(1:2:n+2));
    TE2(2:l,:) = TE2(2:l,:)*sqrt(2);
    TO2 = cos(k*argn1(2:2:n+2));
    TO2(2:l,:) = TO2(2:l,:)*sqrt(2);
    % even,even moments matrix
    mom = 2*sqrt(2)./(1-k.^2);
    mom(1) = 2;
    [M1,M2] = meshgrid(mom);
    M = M1.*M2;
    Mmom = fliplr(triu(fliplr(M)));
    % interpolation weights matrices
    W1 = 2*ones(l)/(n*(n+1));
    W2 = 2*ones((n+mod(n,2))/2+1,(n+mod(n,2))/2)/(n*(n+1));
    W1(:,1) = W1(:,1)/2;
    W2(1,:) = W2(1,:)/2;
    if (mod(n,2) == 0)
        %  Mmom(n/2+1,1) = Mmom(n/2+1,1)/2;
        Mmom(1,n/2+1) = Mmom(1,n/2+1)/2;
        W1(:,n/2+1) = W1(:,n/2+1)/2;
        W1(n/2+1,:) = W1(n/2+1,:)/2;
    else
        W2((n+1)/2+1,:) = W2((n+1)/2+1,:)/2;
        W2(:,(n+1)/2) = W2(:,(n+1)/2)/2;
    end
    % cubature weights as matrices on the subgrids.
    L1 = W1.*(TO2'*Mmom*TE1);
    L2 = W2.*(TE2'*Mmom*TO1);
    if (mod(n,2) == 0)
        L = zeros(n/2+1,n+1);
        L(:,1:2:n+1) = L1;
        L(:,2:2:n+1) = L2;
        L = L(:);
    else
        L = zeros((n+1)/2,(n+2));
        L = [L1',L2']';
        L = L(:);
    end
    L = L*(xyrange(2)-xyrange(1))*(xyrange(4)-xyrange(3))/4;
end
if (nargout == 0 | nargout == 1)
    varargout{1} = L;
else
    varargout{1} = L1;
    varargout{2} = L2;
    varargout{3} = L;
end

function [varargout] = pdpts(n,varargin)

%--------------------------------------------------------------------------
% USAGE of "pdpts".
%
% Pad = pdpts(n)
% Pad = pdpts(n,xyrange)
% [X1,Y1,X2,Y2] = pdpts(n)
% [X1,Y1,X2,Y2] = pdpts(n,xyrange)
%
% Compute the (first family of) Padua points, either as a matrix Pad
% with their abscissas in the first column and their ordinates in the
% second, or as two (mesh)grids X1,Y1 and X2,Y2, respectively.
%--------------------------------------------------------------------------
% INPUT.
%
% n           : interpolation degree
% xyrange     : an optional vector [a,b,c,d] defining the rectangle
%               [a,b] x [c,d]. Otherwise, xyrange = [-1,1,-1,1]
%
% OUTPUT.
%
% Pad         : matrix of size ((n+1)*(n+2)/2) x 2 such
%               that (Pad(:,1),Pad(:,2)) defines the Padua points in the
%               rectangle [xyrange(1),xyrange(2)] x [xyrange(3),xyrange(4)].
% X1,Y1,X2,Y2 : the two (mesh)grids X1,Y1 and X2,Y2 defining the Padua points
%--------------------------------------------------------------------------
% FUNCTIONS CALLED BY THIS CODE:
% no external function is used.
%--------------------------------------------------------------------------

% Copyright (C) 2008-2009
% Marco Caliari, Stefano De Marchi, Alvise Sommariva, Marco Vianello.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%
% Author:
%          Marco Caliari     <marco.caliari@univr.it>
%          Stefano De Marchi <stefano.demarchi@univr.i>
%          Alvise Sommariva  <alvise@euler.math.unipd.it>
%          Marco Vianello    <marcov@euler.math.unipd.it>
%
% Date: March 04, 2009.
%--------------------------------------------------------------------------

if (nargin == 1)
    % standard square [-1,1] x [-1,1]
    xyrange = [-1,1,-1,1];
else
    % rectangle [xyrange(1),xyrange(2)] x [xyrange(3),xyrange(4)]
    xyrange = varargin{1};
end
if (n == 0)
    % degree 0
    if (nargout ~= 4)
        % points as a single matrix
        Pad = [xyrange(1),xyrange(3)];
        varargout = {Pad};
    else
        % points as two (mesh)grids
        X1 = xyrange(1);
        Y1 = xyrange(3);
        X2 = zeros(1,0);
        Y2 = zeros(1,0);
        varargout = {X1,Y1,X2,Y2};
    end
else
    % degree > 0
    zn = (xyrange(1)+xyrange(2)+(xyrange(2)-xyrange(1))*...
        cos(linspace(0,1,n+1)*pi))/2;
    zn1 = (xyrange(3)+xyrange(4)+(xyrange(4)-xyrange(3))*...
        cos(linspace(0,1,n+2)*pi))/2;
    if (nargout ~= 4)
        % points as a single matrix
        [Pad1,Pad2] = meshgrid(zn,zn1);
        [M1,M2] = meshgrid([0:n],[0:n+1]);
        findM = find(mod(M1+M2,2));
        Pad = [Pad1(findM),Pad2(findM)];
        varargout = {Pad};
    else
        % points as two (mesh)grids
        En = zn(1:2:n+1);
        On = zn(2:2:n+1);
        En1 = zn1(1:2:n+2);
        On1 = zn1(2:2:n+2);
        [X1,Y1] = meshgrid(En,On1);
        [X2,Y2] = meshgrid(On,En1);
        varargout = {X1,Y1,X2,Y2};
    end
end


%----------------------------------------------------------------------
% 3. "points2distances". USED BY "autorotation".
%----------------------------------------------------------------------

function distances = points2distances(points)

% Create euclidean distance matrix from point matrix.

% Get dimensions.
[numpoints,dim]=size(points);

% All inner products between points.
distances=points*points';

% Vector of squares of norms of points.
lsq=diag(distances);

% Distance matrix.
distances=sqrt(repmat(lsq,1,numpoints)+repmat(lsq,1,numpoints)'-...
    2*distances);


function [nodes,weights]=quadrature_rules_1D(n,cubature_type)

% SEE WALDVOGEL PAPER. ADDED NODES

% Weights of the Fejer2, Clenshaw-Curtis and Fejer1 quadrature by DFTs
% n>1. Nodes: x_k = cos(k*pi/n)

switch cubature_type
    case 1 % FEJER 1.
        N=[1:2:n-1]'; l=length(N); m=n-l; K=[0:m-1]';
        v0=[2*exp(i*pi*K/n)./(1-4*K.^2); zeros(l+1,1)];
        v1=v0(1:end-1)+conj(v0(end:-1:2));
        weights=ifft(v1);
        k=(1/2):(n-(1/2)); nodes=(cos(k*pi/n))';

    case 2 % FEJER 2.
        N=[1:2:n-1]'; l=length(N); m=n-l; K=[0:m-1]';
        v0=[2./N./(N-2); 1/N(end); zeros(m,1)];
        v2=-v0(1:end-1)-v0(end:-1:2);
        wf2=ifft(v2); weights=[wf2;0];
        k=0:n; nodes=(cos(k*pi/n))';

    case 3 % CLENSHAW-CURTIS.
        n=n-1;
        N=[1:2:n-1]'; l=length(N); m=n-l; K=[0:m-1]';
        g0=-ones(n,1); g0(1+l)=g0(1+l)+n; g0(1+m)=g0(1+m)+n;
        g=g0/(n^2-1+mod(n,2));
        v0=[2./N./(N-2); 1/N(end); zeros(m,1)];
        v2=-v0(1:end-1)-v0(end:-1:2);
        wcc=ifft(v2+g); weights=[wcc;wcc(1,1)];
        k=0:n; nodes=(cos(k*pi/n))';

    case 4 % GAUSS-LEGENDRE
        beta=0.5./sqrt(1-(2*(1:n)).^(-2));
        T=diag(beta,1)+diag(beta,-1);
        [V,D]=eig(T);
        x=diag(D); [x,index]=sort(x); x=x';
        w=2*V(1,index).^2;
        nodes=x';
        weights=w';

    case 5
        % GAUSS-LEGENDRE-LOBATTO.
        xw=lobatto_jacobi(n,0,0);
        nodes=xw(:,1);               % NODI GAUSS-LEGENDRE [-1,1].
        weights=xw(:,2);

end




%--------------------------------------------------------------------------
% R_JACOBI Recurrence coefficients for monic Jacobi polynomials.
%
%    ab=R_JACOBI(n,a,b) generates the first n recurrence 
%    coefficients for monic Jacobi polynomials with parameters 
%    a and b. These are orthogonal on [-1,1] relative to the
%    weight function w(t)=(1-t)^a(1+t)^b. The n alpha-coefficients
%    are stored in the first column, the n beta-coefficients in
%    the second column, of the nx2 array ab. The call ab=
%    R_JACOBI(n,a) is the same as ab=R_JACOBI(n,a,a) and
%    ab=R_JACOBI(n) the same as ab=R_JACOBI(n,0,0).
%
%    Supplied by Dirk Laurie, 6-22-1998; edited by Walter
%    Gautschi, 4-4-2002.
%--------------------------------------------------------------------------
function ab=r_jacobi(N,a,b)

nu=(b-a)/(a+b+2);
mu=2^(a+b+1)*gamma(a+1)*gamma(b+1)/gamma(a+b+2);
if N==1
 ab=[nu mu]; return
end

N=N-1;
n=1:N;
nab=2*n+a+b;
nuadd=(b^2-a^2)*ones(1,N)./(nab.*(nab+2));
A=[nu nuadd];
n=2:N;
nab=nab(n);
B1=4*(a+1)*(b+1)/((a+b+2)^2*(a+b+3));
B=4*(n+a).*(n+b).*n.*(n+a+b)./((nab.^2).*(nab+1).*(nab-1));
abadd=[mu; B1; B'];
ab=[A' abadd];




%--------------------------------------------------------------------------
% GAUSS Gauss quadrature rule.
%
%    Given a weight function w encoded by the nx2 array ab of the 
%    first n recurrence coefficients for the associated orthogonal
%    polynomials, the first column of ab containing the n alpha-
%    coefficients and the second column the n beta-coefficients, 
%    the call xw=GAUSS(n,ab) generates the nodes and weights xw of
%    the n-point Gauss quadrature rule for the weight function w.
%    The nodes, in increasing order, are stored in the first 
%    column, the n corresponding weights in the second column, of
%    the nx2 array xw.
%--------------------------------------------------------------------------
function xw=gauss(N,ab)
N0=size(ab,1); if N0<N, error('input array ab too short'), end
J=zeros(N);
for n=1:N, J(n,n)=ab(n,1); end
for n=2:N
  J(n,n-1)=sqrt(ab(n,2));
  J(n-1,n)=J(n,n-1);
end
[V,D]=eig(J);
[D,I]=sort(diag(D));
V=V(:,I);
xw=[D ab(1,2)*V(1,:)'.^2];



%--------------------------------------------------------------------------
% LOBATTO_JACOBI  Gauss-Lobatto quadrature rule for Jacobi weight 
% function.
%
%    xw=LOBATTO_JACOBI(n,a,b) generates the (n+2)-point Gauss-
%    Lobatto rule for the Jacobi weight function on [-1,1] with
%    parameters a and b. The n+2 nodes are stored in the first
%    column, the n+2 weights in the second column, of the (n+2)x2
%    array xw. The call xw=LOBATTO_JACOBI(n,a) is the same as xw=
%    LOBATTO_JACOBI(n,a,a) and xw=LOBATTO_JACOBI(n) the same as
%    xw=LOBATTO_JACOBI(n,0,0). 
%
%    REFERENCE: W. Gautschi,``High-order Gauss-Lobatto formulae'',
%    Numer. Algorithms 25 (2000), 213-222.
%--------------------------------------------------------------------------
function xw=lobatto_jacobi(N,a,b)
if nargin<2, a=0; end; if nargin<3, b=a; end
ab=r_jacobi(N+2,a,b);
ab(N+2,1)=(a-b)/(2*N+a+b+2);
ab(N+2,2)=4*(N+a+1)*(N+b+1)*(N+a+b+1)/((2*N+a+b+1)* ...
(2*N+a+b+2)^2);
xw=gauss(N+2,ab);




%--------------------------------------------------------------------------
% KRONROD Gauss-Kronrod quadrature formula.
%
%    xw=KRONROD(n,ab) generates the (2n+1)-point Gauss-Kronrod
%    quadrature rule for the weight function w encoded by the 
%    recurrence matrix ab of order [ceil(3*n/2)+1]x2 containing
%    in its first and second column respectively the alpha- and 
%    beta-coefficients in the three-term recurrence relation
%    for w. The 2n+1 nodes, in increasing order, are output
%    into the first column, the corresponding weights into the
%    second column, of the (2n+1)x2 array xw.
%
%    Supplied by Dirk Laurie, 6-22-1998
%--------------------------------------------------------------------------
function xw=kronrod(N,ab)
ab0=r_kronrod(N,ab);
if(sum((ab0(:,2)>0))~=2*N+1) error('Gauss-Kronrod does not exist'), end
J=zeros(2*N+1);
for k=1:2*N
  J(k,k)=ab0(k,1);
  J(k,k+1)=sqrt(ab0(k+1,2));
  J(k+1,k)=J(k,k+1);
end
J(2*N+1,2*N+1)=ab0(2*N+1,1);
[V,D]=eig(J);
d=diag(D);
e=ab0(1,2).*(V(1,:).^2);
[x,i]=sort(d);
w=e(i)';
xw=[x w];





%--------------------------------------------------------------------------
% R_KRONROD Jacobi-Kronrod matrix.
%
%    ab=R_KRONROD(n,ab0) produces the alpha- and beta-elements in 
%    the Jacobi-Kronrod matrix of order 2n+1 for the weight
%    function (or measure) w. The input data for the weight
%    function w are the recurrence coefficients of the associated
%    orthogonal polynomials, which are stored in the array ab0 of
%    dimension [ceil(3*n/2)+1]x2. The alpha-elements are stored in
%    the first column, the beta-elements in the second column, of 
%    the (2*n+1)x2 array ab.
%
%    Supplied by Dirk Laurie, 6-22.1998
%--------------------------------------------------------------------------
function ab=r_kronrod(N,ab0)
if length(ab0)<ceil(3*N/2)+1, error('array ab0 too short'), end
a=zeros(2*N+1,1); b=a;
k=0:floor(3*N/2); a(k+1)=ab0(k+1,1);
k=0:ceil(3*N/2); b(k+1)=ab0(k+1,2);
s=zeros(floor(N/2)+2,1); t=s; t(2)=b(N+2);
for m=0:N-2,
  k=floor((m+1)/2):-1:0; l=m-k;
  s(k+2)=cumsum((a(k+N+2)-a(l+1)).*t(k+2)+b(k+N+2).*s(k+1)-b(l+1).*s(k+2));
  swap=s; s=t; t=swap;
end
j=floor(N/2):-1:0; s(j+2)=s(j+1);
for m=N-1:2*N-3,
  k=m+1-N:floor((m-1)/2); l=m-k; j=N-1-l;
  s(j+2)=cumsum(-(a(k+N+2)-a(l+1)).*t(j+2)-b(k+N+2).*s(j+2)+b(l+1).*s(j+3));
  j=j(length(j)); k=floor((m+1)/2);
  if rem(m,2)==0, a(k+N+2)=a(k+1)+(s(j+2)-b(k+N+2)*s(j+3))/t(j+3);
  else b(k+N+2)=s(j+2)/s(j+3);
  end
  swap=s; s=t; t=swap;
end
a(2*N+1)=a(N)-b(2*N+1)*s(2)/t(2);
ab=[a b];





function [nodes_x, nodes_y, weights]=cubature_manager(M,p_f,p_g,...
    cubature_type)

%--------------------------------------------------------------------------
% INPUTS:
%----------
%
% M: degree of precision required by the rule.
% p_f, p_g: polynomial degrees used by chebfun to approximate as (f,g) a
%           part of the boundary (panel).
% cubature_type: 0: PADUA POINTS.
%                1: FEJER 1 TYPE (TENSORIAL).
%                2: FEJER 2 TYPE (TENSORIAL).
%                3: CLENSHAW-CURTIS (TENSORIAL).
%                4: GAUSS-LEGENDRE (TENSORIAL).
%                5: GAUSS-LEGENDRE-LOBATTO (TENSORIAL).
%--------------------------------------------------------------------------
% OUTPUTS:
%----------
%
% nodes_x, nodes_y, weights: NODES AND WEIGHTS OF A TENSORIAL RULE WITH
% DEGREE OF PRECISION "M" ON THE UNIT SQUARE [-1,1]x[-1,1]. THEY ARE COLUMN
% VECTORS.
%--------------------------------------------------------------------------

switch cubature_type
    case 4
        % lower degree of a Gauss-Legendre rule with ADE equal to M.
        Ntau=floor(M/2)+1; 
        % In the quadrat. rule, for M as input we have a rule with M+1 pts.
        Ntau_cub=Ntau-1; 
        [nodes_tau,weights_tau]=quadrature_rules_1D(Ntau_cub,cubature_type);
        
        actual_ADE=2*length(weights_tau)-1;
        Nu_deg=actual_ADE*max(p_f,p_g)+p_f+p_g-1;
        % lower degree of a Gauss-Legendre rule with ADE equal to Nu_deg.
        Nu=floor(Nu_deg/2)+1; 
        Nu_cub=Nu-1;
        [nodes_u,weights_u]=quadrature_rules_1D(Nu_cub,cubature_type);

        [nodes_x,nodes_y]=meshgrid(nodes_tau,nodes_u);
        nodes_x=nodes_x(:);
        nodes_y=nodes_y(:);

        [weights1,weights2]=meshgrid(weights_tau,weights_u);
        weights=weights1(:).*weights2(:);
    
    case 5
        % lower degree of a Gauss-Legendre-Lobatto rule with ADE equal to M.
        Ntau=ceil((M-1)/2); 
        % In the quadrat. rule, for M as input we have a rule with M+1 pts.
        Ntau_cub=Ntau-1; 
        [nodes_tau,weights_tau]=quadrature_rules_1D(Ntau_cub,cubature_type);
        
        actual_ADE=2*length(weights_tau)-1;
        Nu_deg=actual_ADE*max(p_f,p_g)+p_f+p_g-1;
        % lower degree of a Gauss-Legendre rule with ADE equal to Nu_deg.
        Nu=floor(Nu_deg/2)+1; 
        Nu_cub=Nu-1;
        [nodes_u,weights_u]=quadrature_rules_1D(Nu_cub,cubature_type);

        [nodes_x,nodes_y]=meshgrid(nodes_tau,nodes_u);
        nodes_x=nodes_x(:);
        nodes_y=nodes_y(:);

        [weights1,weights2]=meshgrid(weights_tau,weights_u);
        weights=weights1(:).*weights2(:);    
    
        
    case 0
        % Degree of Padua points for achieving degree of precision "N".
        Ntau=M;
        Nu_deg=M*max(p_f,p_g)+p_f+p_g-1;;
        Nu=Ntau;
        % Computing Padua points and weights in the square [-1,1]x[-1,1]
        PadL = pdcub(Ntau,[-1 1 -1 1]);
        % Working with vectors.
        nodes_x=PadL(:,1); nodes_y=PadL(:,2); weights=PadL(:,3);
        
    otherwise
        Ntau=M; % lower degree of a rule with ADE equal to M.
        [nodes_tau,weights_tau]=quadrature_rules_1D(Ntau,cubature_type);
        
        % lower degree of a rule with ADE equal to Nu_deg=(M+2)*p_i-1.
        Nu_deg=M*max(p_f,p_g)+p_f+p_g-1;; 

        [nodes_u,weights_u]=quadrature_rules_1D(Nu_deg,cubature_type);

        [nodes_x,nodes_y]=meshgrid(nodes_tau,nodes_u);
        nodes_x=nodes_x(:);
        nodes_y=nodes_y(:);

        [weights1,weights2]=meshgrid(weights_tau,weights_u);
        weights=weights1(:).*weights2(:);
end

%--------------------------------------------------------------------------
% 1. "auto_rotation"
%--------------------------------------------------------------------------
function [polygon_bd_rot,rot_matrix,rot_angle,axis_abscissa,vertex_1,...
    vertex_2]=auto_rotation(polygon_bd,vertex_1,vertex_2)

%--------------------------------------------------------------------------
% AUTOMATIC ROTATION OF A CONVEX POLYGON SO THAT "GAUSSIAN POINTS", AS IN 
% THE PAPERS:
%
% 1. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-like and triangulation-free cubature over polygons".
%
% 2. A. SOMMARIVA and M. VIANELLO 
%    "Gauss-Green cubature over spline curvilinear polygons"
%
% THEY ARE ALL (OR "MOST OF THEM" IF THE DEGREE OF THE BORDER IS BIGGER 
% THAN 2) CONTAINED IN THE CONVEX POLYGON. SEE THE PAPERS FOR DETAILS.
%--------------------------------------------------------------------------

% FIND DIRECTION AND ROTATION ANGLE.
if length(vertex_1) == 0
    % COMPUTING ALL THE DISTANCES BETWEEN POINTS.A LITTLE TIME CONSUMING 
    % AS PROCEDURE.
    distances = points2distances(polygon_bd);
    [max_distances,max_col_comp]=max(distances,[],2);
    [max_distance,max_row_comp]=max(max_distances,[],1);
    vertex_1=polygon_bd(max_col_comp(max_row_comp),:);
    vertex_2=polygon_bd(max_row_comp,:);
    direction_axis=(vertex_2-vertex_1)/max_distance;
else
    direction_axis=(vertex_2-vertex_1)/norm(vertex_2-vertex_1);
end

rot_angle_x=acos(direction_axis(1));
rot_angle_y=acos(direction_axis(2));

if rot_angle_y <= pi/2
    if rot_angle_x <= pi/2
        rot_angle=-rot_angle_y;
    else
        rot_angle=rot_angle_y;
    end
else
    if rot_angle_x <= pi/2
        rot_angle=pi-rot_angle_y;
    else
        rot_angle=rot_angle_y;
    end
end


% CLOCKWISE ROTATION.
rot_matrix=[cos(rot_angle) sin(rot_angle);
    -sin(rot_angle) cos(rot_angle)];

number_sides=size(polygon_bd,1)-1;

polygon_bd_rot=(rot_matrix*polygon_bd')';

axis_abscissa=rot_matrix*vertex_1';
